/************************************************************************
 * Programa    : mceitor 0.9.2 			         		*
 * Archivo     : mccurses.h  						*
 * Descripcion : Declaraciones de funciones, estructuras y constantes 	*
 * usadas por mccurses.c 						*
 * 									*
 * Autor : Boris Estudiez						*
 * Nick  : slice							*
 * Fecha : 11/11/2001 							*
 * Mails : 43824@electronica.frc.utn.edu.ar - boris@flashmail.com 	*
 *	 								*
 * Licenced Under GNU							*
 *	 								*
 * Powered by C!							*
 ************************************************************************/

typedef struct wd_menu 
{
   WINDOW *win; /* Ventana "padre", no puede ser subwin*/
   WINDOW *menu_win; /* Sub-Ventana externa del menu */
   WINDOW *menu_subwin; /* Sub-Ventana interna del menu (items) */
   WINDOW *menu_txtwin; /* Sub-Ventaba donde aparece un texto */ 
   MENU *menu; /* Menu */
   ITEM **menu_items; /* Arreglo de punteros a items */
   chtype menu_win_color; /*Color de la ventana externa*/
   chtype menu_item_fore; /* Color principal de los items */
   chtype menu_item_back;/* Color de fondo de los items */
   chtype menu_txt_attr;/* Atributo del texto adicional */
   char **item_name; /* Arreglo de punteros a nombres de los items */
   char *menu_title; /* Titulo del menu o NULL */ 
   char *menu_txt; /* Texto adicional (max 2 lineas) o NULL */
   char *menu_mark; /* Carcarter comodin, para los items */
   char *menu_format; /* Formato HORIZONTAL o VERTICAL */
   char *menu_item_justify; /* CENTER o LEFT */
   int n_items; /* Numero de items que contiene el menu */
   int pos_y; /* Posicion en y  */
   int pos_x; /* Posicion en x  */
   int (*funcion)(struct wd_menu *, char *action, int param); 
   /*Funcion que usara driver_wizard_menu en c/accion, primer parametro 
   estructura del menu wizard, segundo parametro accion llevada que puede ser
   "ENTER" o "MOVE". Debe ser NULL si no se pasa funcion. Retornar
   OK para exito y ERR para error. El tercer parametro se puede definir en 
   cualquier momento para controlar a la funcion.*/   
   int funcion_param; /* parametro adicional que se le pasara a la funcion */
   bool menu_lock; /* TRUE, menu en uso. !TRUE menu libre */
   bool menu_posted; /* TRUE o FALSE */
   bool menu_txt_center; /* TRUE o FALSE */
   bool pos_x_center; /* Centrar menu respecto a x en win, TRUE o FALSE */
   
}  WIZARD_MENU; 

typedef struct mx_form
{

   WINDOW *win; /* Ventana "padre", no puede ser subwin */
   WINDOW *matrix_win; /* Sub-Ventana externa de la matriz */
   WINDOW *matrix_subwin; /* Sub-Ventana de los campos */
   FORM *form_matrix; /* Formulario */
   FIELD **field_matrix; /* arreglo de punteros a campos */
   chtype matrix_field_fore; /* Color de campo */
   chtype matrix_field_back; /* Color de fondo de campo */
   chtype matrix_field_tind_fore; /* Color de campo de los terminos ind. */
   chtype matrix_field_tind_back; /* Color de fondo de campo de term. ind.*/
   chtype matrix_win_color; /* Color de la ventana externa de la matriz */
   chtype matrix_subwin_color; /* Color de la ventana interna de la matriz*/
   char *matrix_style; /* "NORMAL", "SE" (Sistema de ecuaciones), "RESULT" */
   int row; /* Cantidad de filas */
   int col; /* Cantidad de columnas */
   int field_deep; /* Profundidad de campo */
   int field_space; /* Espacio entre campos */
   int pos_y; /* Posicion en y  */
   int pos_x; /* Posicion en x  */
   bool form_lock; /* TRUE, form en uso. !TRUE menu libre */
   bool form_posted; /* TRUE o FALSE */
   bool matrix_edit; /* TRUE (puede ser editada) o  FALSE */

}  MATRIX_FORM;


typedef struct win_buffer_line
{ 
   WINDOW *win; /* ventana asociada al buffer */
   char **winbuf; /* ptrs. a lineas del buffer, el ultimo ptr. debe ser NULL*/ 
   bool own_winbuf; /* buffer propio? TRUE o FALSE */
   int wbufsize; /* tamanio del buffer */
}  WINBUF;  


/* 
   Contantes para pares de Colores, F=Fore_Color, B=Back_Color
   BK=BLACK, BE=BLUE
*/

#define COLOR_FWBR   10
#define COLOR_FWBBE  11
#define COLOR_FCBBE  12
#define COLOR_FBKBW  13                                                  
#define COLOR_FCBW   14
#define COLOR_FRBW   15
#define COLOR_FWBBK  16
#define COLOR_FRBBK  17
#define COLOR_FGBBK  18
#define COLOR_FWBC   19
#define COLOR_FRBC   20
#define COLOR_FMBBK  21
#define COLOR_FYBBK  22
#define COLOR_FCBBK  23
#define COLOR_FBEBBK 24
              	

/* 
   Funciones que devuelven enteros: ERR = error, OK = operacion exitosa 
   Funciones que devuelven punteros : NULL para error 
*/

/* Funciones para formularios de matrices tipo MATRIX_FORM */
int new_matrix_form(MATRIX_FORM *mx_f);
int del_matrix_form(MATRIX_FORM *mx_f);
int load_matrix_form(MATRIX_FORM *mx_f);
int redraw_matrix_form(MATRIX_FORM *mx_f);
int matf2mat(MATRIX *mat, MATRIX_FORM *mx_f);
int matfse2mat(MATRIX *coef, MATRIX *t_ind, MATRIX_FORM *mx_f);
int mat2matf(MATRIX_FORM *mx_f, MATRIX *mat);


/* Funciones para menus tipo WIZARD_MENU */
int  new_wizard_menu(WIZARD_MENU *wd_m);
int  del_wizard_menu(WIZARD_MENU *wd_m);
int  sn_wizard_menu(WINDOW *win, int pos_y, int pos_x, char *msg, char *txt,
     		    chtype txt_attr);
int  ok_wizard_menu(WINDOW *win, int pos_y, int pos_x, char *msg, char *txt,
   		    chtype txt_attr);
char *drive_wizard_menu(WIZARD_MENU *wd_m);
char *wizard_menu(WIZARD_MENU *wd_m);


/* Funciones para buffer's tipo WINBUF */
WINBUF *new_winbuf(WINDOW *win, int buf_size, char **buffer);
void   del_winbuf(WINBUF *wbuf);
void   print_winbuf(WINBUF *wbuf, int a, int b);
char   drive_winbuf(WINBUF *wbuf);

