/************************************************************************
 * Programa    : mceitor 0.9.2 			         		*
 * Archivo     : mccurses.c  						*
 * Descripcion : Funciones de creacion y manejo de: menus (WIZARD_MENU) * 
 * , formularios de matrices (MATRIX_FORM) y buffer's (WINBUF) escritos *
 * con la libreria NCURSES.						*
 * 									*
 * Autor : Boris Estudiez						*
 * Nick  : slice							*
 * Fecha : 11/11/2001 							*
 * Mails : 43824@electronica.frc.utn.edu.ar - boris@flashmail.com 	*
 * Web	 : http://geocities.com/slicesoft				*
 *	 								*
 * Licenced Under GNU							*
 *	 								*
 * Powered by C!							*
 ************************************************************************/


#include <curses.h>
#include <menu.h>
#include <form.h>
#include <string.h>
#include <stdlib.h>
#include "include/mxlib.h"
#include "include/mccurses.h"

struct msg /* declarada y definida en include/mceitor.h */
{
   char *main_menu[10];
   char *seccion_titles[5];
   char *misc_men[13];
   char *misc_txt[14];
};

extern struct msg *msg_lan;

/***************************************************************************
 * new_wizard_menu: Crea un menu a partir de los parametros pasados en una *
 * variable tipo WIZARD_MENU.						   *
 * Retorna: ERR = error, OK = operacion exitosa .			   *
 ***************************************************************************/
int new_wizard_menu(WIZARD_MENU *wd_m)
{
   
   int title_long=0, txt_long=0, i, x, y;
   int k_size; /* tamanio adicional en x de alguna ventana*/
   int x_size, y_size; /* Tamanio del menu */
   int wintxt_sx=0, wintxt_sy=0; /* Tamanio de la ventana de texto */
   int win_sx, win_sy; /* Tamanio de ventana externa */
 
   if(wd_m->win == NULL) /* no exite ventana de referencia */
      return ERR;
   if(wd_m->menu_lock == TRUE) /* menu en uso */
      return ERR;

   /* Creamos Items y Menus, y sus propiedades */
   if((wd_m->menu_items = (ITEM **) calloc(wd_m->n_items+1, sizeof(ITEM *))) 
      == NULL)
      return ERR;
   
   for(i=0; i < wd_m->n_items; i++){
      wd_m->menu_items[i] = new_item(wd_m->item_name[i], NULL);
      if(wd_m->menu_items[i] == NULL)
         return ERR;
   }      
   wd_m->menu_items[i] = NULL; /* Ultimo Item */

   wd_m->menu = new_menu(wd_m->menu_items); /* menu */
   
   if(wd_m->menu == NULL)
      return ERR;
   
   set_menu_back(wd_m->menu, wd_m->menu_item_back);
   set_menu_fore(wd_m->menu, wd_m->menu_item_fore);
   if(strcmp(wd_m->menu_format, "HORIZONTAL") == 0)
      set_menu_format(wd_m->menu, 1, wd_m->n_items);
   if(strcmp(wd_m->menu_format, "VERTICAL") == 0)
      set_menu_format(wd_m->menu, wd_m->n_items, 1);
   set_menu_mark(wd_m->menu, wd_m->menu_mark);
   set_current_item(wd_m->menu, wd_m->menu_items[0]); 
   menu_opts_on(wd_m->menu, O_ROWMAJOR);

   scale_menu(wd_m->menu, &y_size, &x_size); /* Tamanio del menu */
   getmaxyx(wd_m->win, y, x);
 
   if(wd_m->menu_txt != NULL){ /* creacion de la ventana de texto */
      txt_long = strlen(wd_m->menu_txt); /* Tamanio de texto */
      
      if(wd_m->pos_x_center == TRUE && txt_long <= (x-4)-2)
         wd_m->pos_x = x/2 - (txt_long+4)/2 ; /* centrar posicion en x */
         
      if(txt_long <= x-4) 
         wd_m->menu_txtwin = subwin(wd_m->win, 2, txt_long, 
      			     wd_m->pos_y+1, wd_m->pos_x+2);
      else
         wd_m->menu_txtwin = subwin(wd_m->win, 2, x-4, 
      			     wd_m->pos_y+1, wd_m->pos_x+2);

      if(wd_m->menu_txtwin == NULL)
            return ERR;
      
      wbkgd(wd_m->menu_txtwin, wd_m->menu_win_color);
      wattron(wd_m->menu_txtwin, wd_m->menu_txt_attr);  
      wattron(wd_m->menu_txtwin, A_BOLD);  /* por defecto */
      getmaxyx(wd_m->menu_txtwin, wintxt_sy, wintxt_sx);
   }
   else
      wd_m->menu_txtwin = NULL;
      
   if(wd_m->menu_title != NULL)
      title_long = strlen(wd_m->menu_title); /* Tamanio del Titulo */
      
      
   if(title_long >= txt_long)
      k_size = title_long;  
   else
      k_size = wintxt_sx; 
      
   if(k_size > x_size)
      k_size -= x_size; /* diferencia entre k_size y el ancho de menu */
   else
      k_size = 0;
      
   win_sx = x_size+2+2+k_size; /* tamanio de la ventana ext. en x */
   if(wd_m->menu_txt == NULL && wd_m->pos_x_center == TRUE && win_sx <= x-2)
      wd_m->pos_x = x/2 - win_sx/2; /* centrar posicion en x */
 
   /* Sub-Ventanas para menu */

   /*Sub-ventana exterior*/
   wd_m->menu_win = subwin(wd_m->win, y_size+2+wintxt_sy, win_sx, 
      			   wd_m->pos_y, wd_m->pos_x);
   
   if(wd_m->menu_win == NULL)
      return ERR;
      
   getmaxyx(wd_m->menu_win, win_sy, win_sx); /* Tamanio de la ventana ext.*/
   
   /*bkg color de la ventana */
   wbkgd(wd_m->menu_win, wd_m->menu_win_color);
   wattron(wd_m->menu_win, A_BOLD);
   
   wclear(wd_m->menu_win);
   box(wd_m->menu_win, ACS_VLINE, ACS_HLINE);

   if(wd_m->menu_title != NULL)
      mvwaddstr(wd_m->menu_win, 0, win_sx/2 - title_long/2, wd_m->menu_title);

   if(wd_m->menu_txt != NULL) /* imprimir texto */
      mvwaddstr(wd_m->menu_txtwin, 0, 0, wd_m->menu_txt);
      
   if(wd_m->menu_txt_center == TRUE) /* localizar al medio texto */
      if(wintxt_sx < x_size)        
         mvwin(wd_m->menu_txtwin, wd_m->pos_y+1, wd_m->pos_x+2+(x_size/2)-
               wintxt_sx/2);
      
   /* Sub-ventana interior*/  
   if(strcmp(wd_m->menu_item_justify, "CENTER") == 0) /* alineamiento */
      wd_m->menu_subwin = subwin(wd_m->win, y_size, x_size, wd_m->pos_y+1+
         			 wintxt_sy, wd_m->pos_x+(win_sx/2)-x_size/2);

   if(strcmp(wd_m->menu_item_justify, "LEFT") == 0) /* alineamiento */  
      wd_m->menu_subwin = subwin(wd_m->win, y_size, x_size, wd_m->pos_y+1
         			 +wintxt_sy, wd_m->pos_x+1);

   if(wd_m->menu_subwin == NULL)
      return ERR;
  
   /* Sub-ventana exterior donde aparece el menu*/
   if(set_menu_win(wd_m->menu, wd_m->menu_win) != E_OK)
      return ERR;
   /* Sub-ventana interior donde aparece el menu (items) */
   if(set_menu_sub(wd_m->menu, wd_m->menu_subwin) != E_OK)
      return ERR;
      
   wd_m->menu_posted = FALSE; /* menu no asociado a las ventanas */
   wd_m->menu_lock = TRUE; /* menu en uso activado */
   keypad(wd_m->menu_subwin, TRUE); 

   return OK;
}


/***************************************************************************
 * del_wizard_menu: Borra un menu creado por new_wizard_menu .		   *
 * Retorna: ERR = error, OK = operacion exitosa.			   *
 ***************************************************************************/ 
int del_wizard_menu(WIZARD_MENU *wd_m)
{
   int i;
            
   if(wd_m->menu_lock != TRUE)
      return ERR; /* No hay menu que borrar */
   
   if(wd_m->menu_posted == TRUE)   
      if(unpost_menu(wd_m->menu) != E_OK)
         return ERR;
   wd_m->menu_posted = FALSE; /* menu unposted */   
   
   if(free_menu(wd_m->menu) != E_OK)
      return ERR;
   for(i=0; i < wd_m->n_items; i++)
      if(free_item(wd_m->menu_items[i]) != E_OK)
         return ERR;
   free(wd_m->menu_items); 
   
   if(wd_m->menu_win==NULL || wd_m->menu_subwin==NULL || wd_m->win==NULL)     
      return ERR;
   
   free(wd_m->item_name);
         
   wbkgd(wd_m->menu_win, getbkgd(wd_m->win)); /* tomar color de wd_m->win */
   wbkgd(wd_m->menu_subwin, getbkgd(wd_m->win));
   wbkgd(wd_m->menu_txtwin, getbkgd(wd_m->win));
         
      
   wclear(wd_m->menu_win);
   wclear(wd_m->menu_subwin);
   wclear(wd_m->menu_txtwin);
   touchwin(wd_m->win);
   wnoutrefresh(wd_m->menu_txtwin);
   wnoutrefresh(wd_m->menu_win);
   wnoutrefresh(wd_m->menu_subwin);
   doupdate();
   delwin(wd_m->menu_win);
   delwin(wd_m->menu_subwin);
   delwin(wd_m->menu_txtwin);
   
   
   wd_m->menu_lock = FALSE; /* Bloqueo desactivado */
 
   return OK;
}   


/***************************************************************************
 * drive_wizard_menu: Funcion para manejar el menu.			   *
 * Retorna: NULL = error, o la direccion del nombre del objeto (item)      *
 * seleccionado con ENTER o '\n'.					   *
 ***************************************************************************/ 
char *drive_wizard_menu(WIZARD_MENU *wd_m)
{
   int c=0;
   
   if(wd_m->menu_lock != TRUE)
      return NULL; /* No hay menu para manejar */
   
   /* Asociar menu */
   if(wd_m->menu_posted == FALSE){
      if(post_menu(wd_m->menu) != E_OK) /* postearlo */
         return NULL;
      wd_m->menu_posted = TRUE;
  }         

   /* Descomentar si quiere llamar (*wd_m->funcion) desde el primer momento 
   if(wd_m->funcion != NULL && (*wd_m->funcion)(wd_m, "MOVE", 
   wd_m->funcion_param) != OK)
      return NULL;
   */   
  
   while(1){
      touchwin(wd_m->win);
      wnoutrefresh(wd_m->menu_win);
      wnoutrefresh(wd_m->menu_subwin);
      wnoutrefresh(wd_m->menu_txtwin);
      doupdate();   
   	
        
      c = wgetch(wd_m->menu_subwin);  

      if(c == KEY_LEFT){
         menu_driver(wd_m->menu, REQ_LEFT_ITEM);
         if(wd_m->funcion != NULL && (*wd_m->funcion)(wd_m, "MOVE", 
         wd_m->funcion_param) != OK)
            return NULL;
      }      
      if(c == KEY_RIGHT){
         menu_driver(wd_m->menu, REQ_RIGHT_ITEM);
         if(wd_m->funcion != NULL && (*wd_m->funcion)(wd_m, "MOVE", 
         wd_m->funcion_param) != OK)
            return NULL;
      }  
      if(c == KEY_UP){
         menu_driver(wd_m->menu, REQ_UP_ITEM);
         if(wd_m->funcion != NULL && (*wd_m->funcion)(wd_m, "MOVE", 
         wd_m->funcion_param) != OK)
            return NULL;
      }  
      if(c == KEY_DOWN){
         menu_driver(wd_m->menu, REQ_DOWN_ITEM);
         if(wd_m->funcion != NULL && (*wd_m->funcion)(wd_m, "MOVE",
         wd_m->funcion_param) != OK)
            return NULL;
      }
      
      if(c == '\n'){
         if(wd_m->funcion != NULL && (*wd_m->funcion)(wd_m, "ENTER",
         wd_m->funcion_param) != OK)
            return NULL;
         return (char *) item_name(current_item(wd_m->menu)); 
      }
   }
}

/***************************************************************************
 * wizard_menu: Crea (si no existe) y maneja un WIZARD_MENU.		   *
 * Retorna: NULL = error, ptr. al nombre del objeto seleccionado	   *
 ***************************************************************************/ 
char *wizard_menu(WIZARD_MENU *wd_m)
{
   if(wd_m->menu_lock != TRUE) /* no existe el menu */
      if(new_wizard_menu(wd_m) == ERR) /* crear el menu */
         return NULL;
      
   return (char *) drive_wizard_menu(wd_m);
}      

/***************************************************************************
 * ok_wizard_menu: Crea WIZARD_MENU con las opcion Ok.			   *
 * Una ves hecha la eleccion se borra el menu.				   *
 * Si pos_x es < 0, centra el menu en x con respecto a win.		   *
 * Retorna ERR = para error, OK = exito.				   *
 ***************************************************************************/ 
int ok_wizard_menu(WINDOW *win, int pos_y, int pos_x, char *msg, char *txt,
                   chtype txt_attr)
{

   WIZARD_MENU ok_menu;
      
   /* Configuracion de ok_menu */
   ok_menu.win = win;
   ok_menu.menu_item_fore = COLOR_PAIR(COLOR_FBKBW);
   ok_menu.menu_item_back = COLOR_PAIR(COLOR_FWBR);
   ok_menu.menu_win_color = COLOR_PAIR(COLOR_FWBR);
   ok_menu.item_name = (char **) calloc(1, sizeof(char*));
   ok_menu.item_name[0] = msg_lan->misc_men[11];
   ok_menu.n_items = 1;
   ok_menu.menu_title = msg; /* mensage */
   ok_menu.menu_txt = txt; /* texto */
   ok_menu.menu_txt_attr = txt_attr; 
   ok_menu.menu_txt_center = TRUE;
   ok_menu.menu_mark = NULL;
   ok_menu.menu_format = "HORIZONTAL";
   ok_menu.menu_item_justify = "CENTER";
   ok_menu.pos_y = pos_y;
   ok_menu.menu_lock = FALSE;
   
   if(pos_x < 0)
      ok_menu.pos_x_center = TRUE;
   else    
      ok_menu.pos_x = pos_x;   
   ok_menu.funcion = NULL;
   
   if(wizard_menu(&ok_menu) == NULL)
      return ERR;

   
   if(del_wizard_menu(&ok_menu) == ERR)
      return ERR;
   
   return OK;
}      


/***************************************************************************
 * sn_wizard_menu: Crea WIZARD_MENU con las opciones Si y No.		   *
 * Una ves hecha la eleccion se borra el menu.				   *
 * Si pos_x es < 0, centra el menu en x con respecto a win.		   *
 * Retorna: TRUE = si, FALSE = no . ERR	para error. 			   *
 ***************************************************************************/ 
int sn_wizard_menu(WINDOW *win, int pos_y, int pos_x, char *msg, char *txt,
   		   chtype txt_attr)
{

   WIZARD_MENU sn_menu;
   char *pstr;
      
   /* Configuracion de sn_menu */
   sn_menu.win = win;
   sn_menu.menu_item_fore = COLOR_PAIR(COLOR_FBKBW);
   sn_menu.menu_item_back = COLOR_PAIR(COLOR_FWBR);
   sn_menu.menu_win_color = COLOR_PAIR(COLOR_FWBR);
   sn_menu.item_name = (char **) calloc(2, sizeof(char*));
   sn_menu.item_name[0] = msg_lan->misc_men[9];
   sn_menu.item_name[1] = msg_lan->misc_men[10];
   sn_menu.n_items = 2;
   sn_menu.menu_title = msg; /* mensage */
   sn_menu.menu_txt = txt; /* texto */
   sn_menu.menu_txt_attr = txt_attr; 
   sn_menu.menu_txt_center = TRUE;
   sn_menu.menu_mark = NULL;
   sn_menu.menu_format = "HORIZONTAL";
   sn_menu.menu_item_justify = "CENTER";
   sn_menu.pos_y = pos_y;
   sn_menu.menu_lock = FALSE;
   
   if(pos_x < 0)
      sn_menu.pos_x_center = TRUE;
   else    
      sn_menu.pos_x = pos_x;   
   sn_menu.funcion = NULL;
   
   
   if((pstr = wizard_menu(&sn_menu)) == NULL)
      return ERR;
   if(strcmp(pstr, sn_menu.item_name[0]) == 0){
      if(del_wizard_menu(&sn_menu) == ERR)
         return ERR;
      return TRUE;
   }      
   
   if(del_wizard_menu(&sn_menu) == ERR)
      return ERR;
   return FALSE;
}      

/***************************************************************************
 * new_matrix_form: Crea una matriz, ya sea normal o para sistemas de ec.  *
 * Retorna: ERR = error y OK = Operacion exitosa			   *
 ***************************************************************************/ 
int new_matrix_form(MATRIX_FORM *mx_f)
{
   int x_size, y_size; /* tamanio del formulario */
   int x, y, i;
   int i_margen = 4; /* margen izquierdo */
   int n_fields = mx_f->row * mx_f->col; /* numero de campos */
   
   if(mx_f->form_lock == TRUE) /*Formulario en uso */
      return ERR;
   
   /* Campos para la matriz */
   if((mx_f->field_matrix = (FIELD **) calloc(n_fields+1, sizeof(FIELD *))) 
      == NULL)
      return ERR;
      
   if(strcmp(mx_f->matrix_style, "RESULT") == 0)
      i_margen = 7;
 
   /*Localizar los campos en forma de matriz */
   i=0;
   for(y=0; y < mx_f->row; y++)
      for(x=0; x < (mx_f->col * mx_f->field_deep + mx_f->field_space 
         * mx_f->col); x+=mx_f->field_deep + mx_f->field_space)
         if((mx_f->field_matrix[i++] = new_field(1, mx_f->field_deep, y, x, 0, 
             mx_f->field_deep+1)) == NULL)
             return ERR;
   mx_f->field_matrix[n_fields] = NULL;
   

   /* Modificar propiedades de los campos y crear el formulario asociado */
   
   /* Propiedades de todos los campos */
   for(i=0; i < n_fields; i++){
      set_field_fore(mx_f->field_matrix[i], mx_f->matrix_field_fore);
      set_field_back(mx_f->field_matrix[i], mx_f->matrix_field_back);
      set_field_type(mx_f->field_matrix[i], TYPE_NUMERIC, 5, 0, 5);
      field_opts_off(mx_f->field_matrix[i], O_AUTOSKIP);
      field_opts_off(mx_f->field_matrix[i], O_STATIC);
      if(mx_f->matrix_edit == FALSE )
         field_opts_off(mx_f->field_matrix[i], O_EDIT);
   }   
   
   /* Prop. de Campos de Terminos Independientes (si hay)*/
   if(strcmp(mx_f->matrix_style, "SE") == 0) 
      for(i=mx_f->col-1; i < n_fields; i+=mx_f->col){
         set_field_fore(mx_f->field_matrix[i], mx_f->matrix_field_tind_fore);
         set_field_back(mx_f->field_matrix[i], mx_f->matrix_field_tind_back);
      }   
     
   
   if((mx_f->form_matrix = new_form(mx_f->field_matrix)) == NULL)
      return ERR;

   /* Crear sub-ventanas de el formulario */
   
   scale_form(mx_f->form_matrix, &y_size, &x_size); /*tamanio del formulario*/
  
   mx_f->matrix_win = subwin(mx_f->win, y_size+2, x_size+2+i_margen, 
      		             mx_f->pos_y, mx_f->pos_x); /*ventana externa*/

   if(mx_f->matrix_win == NULL)
      return ERR;
   
   /* bkg color de  ventana externa */   
   wbkgd(mx_f->matrix_win, mx_f->matrix_win_color); 
   
   wclear(mx_f->matrix_win);
    
   mx_f->matrix_subwin = subwin(mx_f->win, y_size, x_size, mx_f->pos_y+1, 
      			        mx_f->pos_x+i_margen); /*ventana interna*/
                                
   if(mx_f->matrix_subwin == NULL)
      return ERR;
   
   /* bkg color de  ventana interna */
   wbkgd(mx_f->matrix_subwin,  mx_f->matrix_subwin_color);
   
   wclear(mx_f->matrix_subwin);
   
   /* Asociar el formulario a ventanas */                                
   if(set_form_win(mx_f->form_matrix, mx_f->matrix_win) != E_OK)
      return ERR;
   if(set_form_sub(mx_f->form_matrix, mx_f->matrix_subwin) != E_OK)
      return ERR;
   
   box(mx_f->matrix_win, ACS_VLINE, ' ');

   /* Colocar indices a la matriz */
   for(y=1; y <= mx_f->row; y++)
      mvwprintw(mx_f->matrix_win, y, 1, "%d", y);  
   
   if(strcmp(mx_f->matrix_style, "NORMAL") == 0)   
      for(x=i_margen, i=1; x-i_margen < (mx_f->col * mx_f->field_deep +
         mx_f->field_space * mx_f->col); x+=mx_f->field_deep + 
         mx_f->field_space, i++)
         mvwprintw(mx_f->matrix_win, 0, x, "%d", i);  
   if(strcmp(mx_f->matrix_style, "SE") == 0){
      for(x=i_margen, i=1; x-i_margen < ((mx_f->col-1) * mx_f->field_deep + 
          mx_f->field_space * (mx_f->col-1)); x+=mx_f->field_deep + 
         mx_f->field_space, i++)
         mvwprintw(mx_f->matrix_win, 0, x, "X(%d)", i);  
      mvwprintw(mx_f->matrix_win, 0, x+mx_f->field_deep/2, "R");    
   }
   if(strcmp(mx_f->matrix_style, "RESULT") == 0){ /*matriz de resultados*/
      for(y=1; y <= mx_f->row; y++)
         mvwprintw(mx_f->matrix_win, y, 1, "X(%d)", y);  
      mvwprintw(mx_f->matrix_win, 0, i_margen+mx_f->field_deep/2, "R");  
   }     
              
   keypad(mx_f->win, TRUE); 
   mx_f->form_lock = TRUE; /* Formulario en uso */
   mx_f->form_posted = FALSE; 

   return OK;
}

/***************************************************************************
 * del_matrix_form: Borra una matriz creada por new_matrix_form.	   *
 * Retorna: ERR = error y OK = Operacion exitosa			   *
 ***************************************************************************/ 
int del_matrix_form(MATRIX_FORM *mx_f)
{

   int i;
   
   if(mx_f->form_lock != TRUE)
      return ERR; /* No hay formulario que borrar */
   
   if(mx_f->form_posted == TRUE)
      if(unpost_form(mx_f->form_matrix) != E_OK)
         return ERR;
   mx_f->form_posted = FALSE;      
         
   if(free_form(mx_f->form_matrix) != E_OK)
      return ERR;
   
   for(i=0; i < mx_f->row * mx_f->col; i++)
      if(free_field(mx_f->field_matrix[i]) != OK)
         return ERR;

   free(mx_f->field_matrix);   
  
   wbkgd(mx_f->matrix_win, getbkgd(mx_f->win)); /*tomar color de mx_f->win*/
   wbkgd(mx_f->matrix_subwin, getbkgd(mx_f->win)); /* idem */
   
   wclear(mx_f->matrix_win);
   wclear(mx_f->matrix_subwin);   
   touchwin(mx_f->win);
   wnoutrefresh(mx_f->matrix_win);
   wnoutrefresh(mx_f->matrix_subwin);
   doupdate();
   delwin(mx_f->matrix_win);
   delwin(mx_f->matrix_subwin);
      
   mx_f->form_lock = FALSE;  /* Bloqueo desactivado */
   
   return OK;   
}

/***************************************************************************
 * load_matrix_form: Funcion para cargar una matriz. 			   *
 * Retorna: ERR = error , OK = operacion exitosa.			   *
 ***************************************************************************/ 
int load_matrix_form(MATRIX_FORM *mx_f)
{
   int STOP=FALSE;
   int ch; /*caracter ingresado por teclado*/
   int last_field = mx_f->row * mx_f->col - 1;
  
     
   if(mx_f->form_lock != TRUE)
      return ERR; /* No hay formulario para manejar */
   
   /* Asociar Formulario */
   if(mx_f->form_posted == FALSE){
      if(post_form(mx_f->form_matrix) != E_OK) /* postearlo */
         return ERR;
       mx_f->form_posted = TRUE;
   }         

   if(pos_form_cursor(mx_f->form_matrix) != E_OK)
      return ERR;
   
   /* Setear primer campo como actual*/
   set_current_field(mx_f->form_matrix, mx_f->field_matrix[0]);
    
   while(STOP==FALSE){
      touchwin(mx_f->win);
      wnoutrefresh(mx_f->matrix_win);
      wnoutrefresh(mx_f->matrix_subwin);
      doupdate();
      
      ch = wgetch(mx_f->win);
      
      if(ch == KEY_LEFT)
         form_driver(mx_f->form_matrix, REQ_LEFT_CHAR);
      else if(ch == KEY_RIGHT)
         form_driver(mx_f->form_matrix, REQ_RIGHT_CHAR);
      else if(ch == KEY_BACKSPACE){
         form_driver(mx_f->form_matrix, REQ_LEFT_CHAR);
         form_driver(mx_f->form_matrix, REQ_DEL_CHAR);
      }      
      else if(ch == KEY_DC) /* delete */
         form_driver(mx_f->form_matrix, REQ_DEL_CHAR);
      else if(ch == KEY_HOME)
         form_driver(mx_f->form_matrix, REQ_BEG_FIELD);
      else if(ch == KEY_END)
         form_driver(mx_f->form_matrix, REQ_END_LINE);
      else if(ch == '\t')  /* mueve al sig. campo */
         form_driver(mx_f->form_matrix, REQ_NEXT_FIELD);  
      else if(ch == '\n'){ /* caracter de validacion */
         form_driver(mx_f->form_matrix, REQ_VALIDATION); /*escribe el bufer*/
         if(current_field(mx_f->form_matrix) 
            == mx_f->field_matrix[last_field])
            break; /* formulario completo */
         form_driver(mx_f->form_matrix, REQ_NEXT_FIELD); /*mueve al sig. campo*/
      }    
      else if(ch == 'e') /* mover al ultimo campo de matriz */
         form_driver(mx_f->form_matrix, REQ_LAST_FIELD);
      else if(ch == 'h') /* mover al primer campo de matriz */
         form_driver(mx_f->form_matrix, REQ_FIRST_FIELD);
      else if(ch == 'q') /* salir de matriz */
         STOP = TRUE;
      else  
         form_driver(mx_f->form_matrix, ch); 
   }
   
   return OK; /*Formulario completo*/
}   


/***************************************************************************
 * matf2mat: Toma los valores de una MATRIX_FORM tipo "NORMAL"		   *
 * y los coloca en una matriz tipo MATRIX.				   *
 * Retorna: ERR = error , OK = operacion exitosa.			   *
 ***************************************************************************/ 
int matf2mat(MATRIX *mat, MATRIX_FORM *mx_f)
{

   int i, j, n;
   
   if(mx_f->form_lock != TRUE) /* No hay formulario en uso */
      return ERR;
 
   if(mx_f->row != mat->row || mx_f->col != mat->col)
      return ERR; /* dimenciones distintas */
      
   if(strcmp(mx_f->matrix_style, "SE") == 0)
      return ERR; /* Es una matriz SE */
   
   /* Volcar datos */  
   n=0;
   for(i=0; i < mat->row; i++)
      for(j=0; j < mat->col; j++) /* solo numeros puede tener el campo */
         mat->val[i][j] = atof(field_buffer(mx_f->field_matrix[n++], 0));
 
   return OK;
}   


/***************************************************************************
 * matfse2mat: Toma los valores de una MATRIX_FORM tipo "SE" 		   *
 * y coloca en dos matrices (ambas tipo MATRIX), los valores de los coefi- *
 * cientes principales y los terminos independientes.			   *
 * Retorna: ERR = error , OK = operacion exitosa.			   *
 ***************************************************************************/ 
int matfse2mat(MATRIX *coef, MATRIX *t_ind, MATRIX_FORM *mx_f)
{ 
   int i, j, n;
   
   if(mx_f->form_lock != TRUE) /* No hay formulario en uso */
      return ERR;

   if(mx_f->row != coef->row || mx_f->col-1 != coef->col)
      return ERR; /* dimenciones distintas */

   if(mx_f->row != t_ind->row || t_ind->col != 1)
      return ERR; /* dimenciones distintas */
     
   if(strcmp(mx_f->matrix_style, "SE") != 0)
      return ERR; /* No es una matriz de SE */

      
   /* Volcar datos a la matriz de coeficientes*/
   n=0;
   for(i=0; i < coef->row; i++){
      for(j=0; j < coef->col; j++)
         coef->val[i][j] = atof(field_buffer(mx_f->field_matrix[n++], 0));
      ++n; /*salteamos la columna de term. ind. */   
   }     
   
   /* Volcar datos al la matriz de terminos independientes*/
   n=mx_f->col-1; /*salteamos coef. principales */
   for(i=0; i < t_ind->row; i++)
      for(j=0; j < t_ind->col; j++){
         t_ind->val[i][j] = atof(field_buffer(mx_f->field_matrix[n], 0));
         n+=mx_f->col; /*salteamos coef. principales*/
      }   

   return OK;

}   

/***************************************************************************
 * mat2fmat: Toma los valores de una MATRIX y los coloca en una matriz tipo* 
 * MATRIX_FORM tipo "NORMAL" o "RESULT".  				   *
 * Retorna: ERR = error , OK = operacion exitosa.			   *
 ***************************************************************************/ 
int mat2matf(MATRIX_FORM *mx_f, MATRIX *mat)
{

   int i, j, n;
   char value[BUFSIZ];
   
   if(mx_f->form_lock != TRUE) /* No hay formulario en uso */
      return ERR;
 
   if(mx_f->row != mat->row || mx_f->col != mat->col)
      return ERR;
      
   if(strcmp(mx_f->matrix_style, "SE") == 0)
      return ERR; /* es una matriz SE */
   
   /* Volcar datos */  
   n=0;
   for(i=0; i < mat->row; i++)
      for(j=0; j < mat->col; j++){
         sprintf(value, "%g", mat->val[i][j]); 
         set_field_buffer(mx_f->field_matrix[n++], 0, value);
      }   

   return OK;
}   

   
/***************************************************************************
 * redraw_matrix_form: Redibuja una MATRIX_FORM. 			   *
 * Usarlo cuando hay solapamiento de ventanas (cosejo).			   *
 * Retorna: ERR = error , OK = operacion exitosa.			   *
 ***************************************************************************/ 
int redraw_matrix_form(MATRIX_FORM *mx_f)
{

   int i, x, y;
   int i_margen = 4; /* margen izquierdo */
   
   if(mx_f->form_lock != TRUE) /* No hay formulario en uso */
      return ERR;
      
   if(strcmp(mx_f->matrix_style, "RESULT") == 0)
      i_margen = 7;   
    
   wclear(mx_f->matrix_win);   
   wclear(mx_f->matrix_subwin);
      
   /* Rescribir el Formulario a su ventana interna */
   if(mx_f->form_posted == FALSE){ /* no esta posteado */
      if(post_form(mx_f->form_matrix) != E_OK) /* postearlo */
         return ERR;
      mx_f->form_posted = TRUE;
   } 
   else {
      unpost_form(mx_f->form_matrix);
      post_form(mx_f->form_matrix); 
   }      

   
   box(mx_f->matrix_win, ACS_VLINE, ' ');

   /* Colocar indices a la matriz */
   for(y=1; y <= mx_f->row; y++)
      mvwprintw(mx_f->matrix_win, y, 1, "%d", y);  
   
   if(strcmp(mx_f->matrix_style, "NORMAL") == 0)   
      for(x=i_margen, i=1; x-i_margen < (mx_f->col * mx_f->field_deep +
          mx_f->field_space * mx_f->col); x+=mx_f->field_deep + 
          mx_f->field_space, i++)
          mvwprintw(mx_f->matrix_win, 0, x, "%d", i);  
   if(strcmp(mx_f->matrix_style, "SE") == 0){
      for(x=i_margen, i=1; x-i_margen < ((mx_f->col-1) * mx_f->field_deep + 
          mx_f->field_space * (mx_f->col-1)); x+=mx_f->field_deep + 
          mx_f->field_space, i++)
          mvwprintw(mx_f->matrix_win, 0, x, "X(%d)", i);  
      mvwprintw(mx_f->matrix_win, 0, x+mx_f->field_deep/2, "R");    
   }
   if(strcmp(mx_f->matrix_style, "RESULT") == 0){ 
      for(y=1; y <= mx_f->row; y++)
         mvwprintw(mx_f->matrix_win, y, 1, "X(%d)", y);  
      mvwprintw(mx_f->matrix_win, 0, i_margen+mx_f->field_deep/2, "R");  
   }     
   
   touchwin(mx_f->win);
   wnoutrefresh(mx_f->matrix_win);
   wnoutrefresh(mx_f->matrix_subwin);
   doupdate();
    
   return OK;
}   
   
   

/* FUNCIONES DE MANEJO DEL WINBUF (En el futuro se mejorara esta funcion)*/

/****************************************************************************
 * new_winbuf:crea un buffer de lineas para una ventana de tamanio buf_size.*
 * Si buffer no es NULL usa tal direccion como buffer, sino crea un buffer  *
 * dinamicamente.							    *
 * Retorna NULL para error o la direccion del winbuf.			    *
 * Siempre el ultimo puntero a char del arreglo winbuf debe ser NULL.	    * ****************************************************************************/

WINBUF *new_winbuf(WINDOW *win, int buf_size, char **buffer)
{
   WINBUF *wbuf;
   char **plines; /* arreglo de punteros a lineas */
   int i;
   
   if((wbuf = (WINBUF *) malloc(sizeof(WINBUF))) == NULL)
      return NULL;
   
   wbuf->win = win;   
   wbuf->wbufsize = buf_size;   
   
   if(buffer == NULL){
      if((wbuf->winbuf = (char **) calloc(buf_size, sizeof(char *))) == NULL)
         return NULL;
      wbuf->own_winbuf = TRUE; /* buffer propio */   
   }      
   else{
      wbuf->winbuf = buffer;
      wbuf->own_winbuf = FALSE; /* buffer ajeno */
   }   
      

   return (WINBUF *) wbuf;
}      



/****************************************************************************
 * print_winbuf: imprime lineas de buffer desde la posicion 'a' hasta 'b'   *
 * respecto al orden del buffer. 					    *	
 ****************************************************************************/   
void print_winbuf(WINBUF *wbuf, int a, int b)
{
   int i, x, y;
     
   wclear(wbuf->win);
   for(i=a; i <= b; i++){
      if(i >=0 && wbuf->winbuf[i] != NULL)
         wprintw(wbuf->win, "%s\n", wbuf->winbuf[i]);
      else
         break;
   }      
}      
   
/****************************************************************************
 * drive_winbuf: Muestra el buffer en su ventana permitiendo desplazarce    *
 * en el con KEY_DOWN y KEY_UP. Retorna al presionar cualquier otra letra   *
 ****************************************************************************/   
char drive_winbuf(WINBUF *wbuf)
{

   int c, x, y, i, buflines=0;

   getmaxyx(wbuf->win, y, x);
   keypad(wbuf->win, TRUE); 
   
   for(i=0; wbuf->winbuf[i] != NULL; i++)
      buflines++; /* cantidad de lineas en el buffer */
   
   print_winbuf(wbuf, 0, y-1);
   
   i=0;
   while(1){
      c = wgetch(wbuf->win);
   
      if(c == KEY_DOWN){
         if(buflines != 0 && buflines-i > y){
            ++i;
            print_winbuf(wbuf, i, y-1+i);
         }   
      }      
      else if(c == KEY_UP){
         if(i != 0){
            --i;
            print_winbuf(wbuf, i, y-1+i);
         }   
      }
      else if(c == KEY_NPAGE){
         if(buflines != 0 && buflines-i > y){
            i+=y;
            print_winbuf(wbuf, i, y-1+i);
         }
      }   
      else if(c == KEY_PPAGE){
         if(i != 0){
            i-=y;
            if(i < 0)
               i=0;
            print_winbuf(wbuf, i, y-1+i);
         }   
      }   
      else
         return c;
   }   
}         
   
/****************************************************************************
 *del_winbuf: borra de memoria un WINBUF 				    *
 ****************************************************************************/
void del_winbuf(WINBUF *wbuf)
{
   int i;
  
  if(wbuf->own_winbuf == TRUE){ /* buffer propio */ 
     for(i=0; wbuf->winbuf[i] != NULL; i++)
        free(wbuf->winbuf[i]);
     free(wbuf->winbuf);
  }   
 
  free(wbuf);
}   



