/************************************************************************
 * mxlib.h     : Manejo y operaciones de Matrices.			*
 * Version     : 0.5  							*
 * Autor       : Boris Estudiez						*
 * Nick        : xdarth							*
 * Fecha       : 11/07/2001 						*
 * Mails: boris@flashmail.com - 43824@electronica.frc.utn.edu.ar	*
 *	 								*
 * Licenced Under GNU							*
 *	 								*
 *	 								*
 * Powered by C!							*
 *  									*
 * Nota: En caso de no haber memoria, algunas funciones abortan el pro- *
 * grama con exit(EXIT_FAILURE) y un mensage en stderr.			*
 *  									*
 ************************************************************************/


/* Constantes Devueltas Por las Funciones */

#define NO_MEMORY -10 /* No hay memoria */
#define NO_SQUARE -9  /* Matriz no cuadrada */
#define OUT_RANGE -8  /* Fuera de rango */
#define E_NULO	  -7  /* Elemento nulo */	
#define NO_FOUND  -6  /* No encontrado */
#define NO_SOL	  -5  /* No tiene solucion */
#define M_ERROR   -1  /* Valor de Error en funcion de matrices */
#define M_OK	   0  /* Todo bien */


/************************************************************************
 *  Dimenciones y puntero de la matriz.					*
 *  Cada matriz creada debe estar dentro de esta estructura.		*
 ************************************************************************/

typedef struct mdat{ int fil; int col; double *matrix; } MATRIX; 


/*------------------------Prototipos de funciones---------------------- */

/* Funciones de manejo de matrices */

double  *dmat(MATRIX *pmat, int fil, int col);
int creatmat(MATRIX *pmat, int fil, int col);
void freemat(MATRIX *pmat);
int check_range(MATRIX *pmat, int *f, int *c);
int colcpy(MATRIX *pm1, MATRIX *pm2, int c1, int c2);
int matcpy(MATRIX *pm1, MATRIX *pm2);
void loadmat(MATRIX *pmat, char *msg);
void printmat(MATRIX *pmat);


/* Funciones Algebraicas y Matematicas */

double trimat(MATRIX *pmat);
double det(MATRIX *pmat);
double mcomp(MATRIX *pmat, int f, int c);
int scandval(MATRIX *pmat, int f1, int col);
int scanuval(MATRIX *pmat, int f1, int col);
int scanrval(MATRIX *pmat, int fil, int c1);
int nullups(MATRIX *pmat, int fila_actual);
int sysresolv(MATRIX *pmA, MATRIX *pmb, MATRIX *result);
int cof(MATRIX *pmat);
int adj(MATRIX *pmat);
int inv(MATRIX *pmat);
void e1(MATRIX *pmat, int f1, int f2);
void e2(MATRIX *pmat, int f1, int f2, double  k);
void e3(MATRIX *pmat, int f1, double  k);
void redmat(MATRIX *pmat);
void trn(MATRIX *pmat);
void multk(MATRIX *pmat, double  k);

/* Macros */

#define vmat(pmat, f, c) *dmat(pmat, f, c) /* Regresa valor del elemento */ 



