/************************************************************************
 * mxlib.c     : Manejo y operaciones de Matrices.			*
 * Version     : 0.5  							*
 * Autor       : Boris Estudiez						*
 * Nick        : xdarth							*
 * Fecha       : 11/07/2001 						*
 * Mails: boris@flashmail.com - 43824@electronica.frc.utn.edu.ar	*
 *	 								*
 * Licenced Under GNU							*
 *	 								*
 * Para Usar La Libreria:						*
 * Headers necesarios: matrixlib.h 					*
 * Librerias que necesita: libreria matematica ( libm )                 *
 * Compilacion: gcc prog.c mxlib.c -lm -o prog                          *
 *	 								*
 * Powered by C!							*
 *  									*
 * Nota: En caso de no haber memoria, algunas funciones abortan el pro- *
 * grama con exit(EXIT_FAILURE) y un mensage en stderr.			*
 *  									*
 * Puntos Flojos: La funcion creatmat tiene que recivir una varible 	*
 * tipo MATRIX, y no una tipo MATRIX *, por lo cual lo unico que crea 	*
 * es la matriz en mat.matrix, pero mat.fil y mat.col, ya estan creados *
 * en la definicion de mat. Ej: MATRIX mat;. Y freemat solo libera	*
 * a mat.matrix. En otra version capas que lo mejoro.			*
 *  									*
 ************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "include/mxlib.h" /* Localizacion de mxlib.h */


/* Variables Globales */
static char buf[BUFSIZ]; /* bufer */



/*------------------FUNCIONES DE MANEJO DE MATRICES---------------------*/

/************************************************************************
 *  creatmat: crea una matriz en memoria de f * c objetos.		*
 *  Devuelve M_ERROR si no se puede crear o M_OK			*
 *  Asigna dimenciones a pmat->fil y pmat->col.				*
 *  Se debe usar esta funcion para crear matrices siempre.		*
 *  f y c deben ser > 0.						*
 ************************************************************************/
   
int creatmat(MATRIX *pmat, int f, int c)
{         
   
   if(f < 0 || c < 0 || f == 0 || c == 0)
      return M_ERROR;
  
   /*Creamos matriz en memoria, en relidad es un arreglo de f * c objetos*/
   if((pmat->matrix = (double *) calloc(f * c, sizeof(double))) == NULL)
      return NO_MEMORY;
   
   /*Guardamos en la estructura pmat las dimenciones de la matriz*/
   pmat->fil = f; 
   pmat->col = c;
   
   return M_OK;
}

/************************************************************************
 * freemat: Libera la memoria ocupada por la matriz en pmat->matrix.    *
 ************************************************************************/
 
void freemat(MATRIX *pmat)
{
   free(pmat->matrix);
}   

/************************************************************************
 * dmat: Regresa la direccion en pmat.matrix de la fila "f" y columna   *
 *       "c" . Se usa igual que matrix[f][c] en el caso de que matrix	*
 * 	 sea una matriz definida como tal.				*
 *	 Devuelve NULL para elemento fuera de rango.			*
 ************************************************************************/

double *dmat(MATRIX *pmat, int f, int c)
{
   if(check_range(pmat, &f, &c) == OUT_RANGE)
      return NULL; /* Fuera de Rango */
   
   if(f == 0 && c == 0)
      return (double *) pmat->matrix;
   else if(f == 0)
      return (double *) (pmat->matrix + c);
   else if(c == 0)
      return (double *) (pmat->matrix + f * pmat->col);
   else
      return (double *) (pmat->matrix + f * pmat->col + c);
}

/************************************************************************
 * NOTA: vmat(MATRIX *, int f, int c) es un macro de dmat y devuelve	*
 * valor en ves de direccion.						*
 ************************************************************************
 
/************************************************************************
 * printmat: imprime matriz, filas y columnas especificadas en la 	*
 * estructura.								*
 ************************************************************************/

void printmat(MATRIX *pmat)
{
   int c, f;
   for(f = 0; f < pmat->fil; f++)   
      for(c = 0; c < pmat->col; c++){
         if(c == 0 && c == (pmat->col - 1)) /*unica columna*/
            printf("|  %-9g |\n", vmat(pmat, f, c)); 
         else if(c == (pmat->col - 1)) /*ultimo elemento de la fila*/
            printf("  %-9g |\n", vmat(pmat, f, c));
         else if(c == 0)  /*primer elemento de fila*/
            printf("| %-9g", vmat(pmat, f, c)); 
         else
            printf("  %-9g", vmat(pmat,  f, c));
      }      
}      


/************************************************************************
 * loadmat: carga con valores la matriz con una etiqueta.		*
 ************************************************************************/

void loadmat(MATRIX *pmat, char *txt)
{ 
   int c, f;
  
   for(f = 0; f < pmat->fil; f++)
      for(c = 0; c < pmat->col; c++){
         printf("%s[%d,%d] = ", txt, f+1, c+1);
         fgets(buf, BUFSIZ, stdin);
         vmat(pmat, f, c) = atof(buf);
      }
}

/************************************************************************
 * colcpy: Copia de m2 la columna c2 a m1 en la col. c1. Ambas tienen   *
 * que tener igual numero de filas.					*
 * Devuelve M_ERROR si hay error o M_OK   de lo contrario.		*
 ************************************************************************/
 
int colcpy(MATRIX *m1, MATRIX *m2, int c1, int c2)
{
   int f;
   
   if(m1->fil != m2->fil) /* distinta cantidad de filas */
      return M_ERROR;
   if(c1 >= m1->col || c2 >= m2->col || c1 < 0 || c2 < 0) /*fuera de rango*/   
      return M_ERROR;
   
   for(f=0; f < m2->fil; f++)
      vmat(m1, f, c1) = vmat(m2, f, c2);
   return M_OK;   
}   

/************************************************************************
 * matcpy: copia pm2 a pm1. Devuelve M_ERROR en caso de tener dimencio	*
 * nes distintas.							*
 ************************************************************************/
 
int matcpy(MATRIX *pm1, MATRIX *pm2)
{

   double temp;
   int c, f;
   
   if(pm1->fil != pm2->fil || pm1->col != pm2->col)
      return M_ERROR; /*dimenciones distintas*/
   
   for(f=0; f < pm2->fil; f++)
      for(c=0; c < pm2->col; c++)
         vmat(pm1, f, c) = vmat(pm2, f, c);
         
}


/************************************************************************
 * check_range: Comprueba si f y c, estan dentro de la matriz.		*
 * Si f o c son NULL, lo/s ignora.					*
 * M_OK, si esta/n dentro de la matriz, OUT_RANGE si no esta/n adentro. *
 ************************************************************************/

int check_range(MATRIX *pmat, int *f, int *c)
{
 
 int val; 
  
 if(f == NULL && c == NULL)
    val =  OUT_RANGE;
 else if(f == NULL)  	/*comprueba si c esta dentro de pmat*/
     ((*c >= 0) && (*c < pmat->col))? (val = M_OK) : (val = OUT_RANGE);
 else if(c == NULL) 	/*comprueba si f esta dentro de pmat*/
    (*f >= 0 && *f < pmat->fil)? (val =  M_OK) : (val = OUT_RANGE);
 else 			/* Comprueba si ambos estan dentro de pmat */    
    ((*c >= 0 && *c < pmat->col) && (*f >= 0 && *f < pmat->fil))? (val=M_OK): (val=OUT_RANGE);
    
 return val;        
}   


/*-----------------------FUNCIONES MATEMATICAS---------------------------*/

/************************************************************************
 * trimat: Triangula una matriz nxn a traves de operaciones elementales.*
 *         Triangular superior, quedaria una matriz cuadrada		*
 * Retorna el producto de todos los escalares usados en las operaciones *
 * elementales de e3 por el producto de tantos -1 como operaciones	*
 * elementales e1 se le hagan a la matriz para trinagularla.            *
 * Retorna 1 en caso de no efectuar operaciones e1 y e3.		*
 * Retorna NO_SQUARE en caso de no ser una matriz cuadrada.		*
 * Nota NO_SQUARE puede coincidir con el regreso normal de la funcion.	* 
 ************************************************************************/

double trimat(MATRIX *pmat)
{
   double k, valor=1;   
   int c, f; /*numero de fila y col. actual */
   int f2;
   
   if(pmat->fil != pmat->col)
      return NO_SQUARE; /*matriz no cuadrada*/
     
   for(c = 0, f = 0; f < pmat->fil && c < pmat->col; f++, c++){
      if(vmat(pmat, f, c) == 0){ /* primer elemento nulo */
         if((f2 = scandval(pmat, f, c)) == E_NULO)
            continue; /* todos los elementos por debajo son nulos */
         else {
            e1(pmat, f, f2); /* f2 no tiene elemento nulo en esta columna*/  
            valor *= -1;
         }   
      }
      if(vmat(pmat, f, c) != 1){ 
         k = vmat(pmat, f, c);
         e3(pmat, f, 1/k); /*convertimos en 1 el elemento de esta col.*/           
         valor *= k;
      }
      if(f < (pmat->fil -1)) /* no estamos en la ultima fila*/
         while((f2 = scandval(pmat, f+1, c)) != E_NULO){ 
            k = (vmat(pmat, f2, c) * -1);
            e2(pmat, f2, f, k);
         }   
   }
   return valor;
}

/*************************************************************************
 * det: calcula el determinate de una matriz. Usa el metodo de triangular*
 * la matriz, luego el determinate asociado a tal matriz es el prod-	 *
 * ucto de la diagonal principal por las propiedades que tienen		 *
 * las operaciones elementales sobre los determinantes.			 *
 * Retorna el valor del determinante.					 *
 * Si la matriz no es cuadrada, el valor de retorno es NO_SQUARE.        *
 *	 								 *
 * Nota: El valor del determinate, puede concidir con NO_SQUARE. 	 *
 *************************************************************************/
    	
double det(MATRIX *pmat)
{
   MATRIX temp; /*copia de la matriz original*/
   double detm;
   int f, c, pos;
   
   if(pmat->col != pmat->fil) /* Matriz no cuadrada, error */
      return NO_SQUARE;
   
   if(creatmat(&temp, pmat->fil, pmat->col) == NO_MEMORY){ 
      fprintf(stderr, "\nNo hay memoria disponible!\n");
      exit(EXIT_FAILURE);
   }   
    
   matcpy(&temp, pmat);  /* copiamos matriz orignal a temporal */    
   detm = trimat(&temp); /* triangulamos matriz y obtenemos prop. de op. elemen.*/   
   
   for(pos = 0; pos < pmat->col; pos++)
      detm *= vmat(&temp, pos, pos); /*producto de la diagonal principal*/

   freemat(&temp); /*liberamos matriz temporal*/   
   
   return detm;
}   


/************************************************************************
 * e1: intercambia f1 por f2 						*
 ************************************************************************/
 
void e1(MATRIX *pmat, int f1, int f2)
{
   double temp;
   int c;
   
   for(c = 0; c < pmat->col; c++){
      temp = vmat(pmat, f1, c);
      vmat(pmat, f1, c) = vmat(pmat, f2, c);
      vmat(pmat, f2, c) = temp;
   }
}

/************************************************************************
 * e2: suma a f1 la f2 previamente multiplicada por k 			*
 ************************************************************************/

void e2(MATRIX *pmat, int f1, int f2, double k)
{
   int c;
   
   for(c = 0; c < pmat->col; c++)
      vmat(pmat, f1, c) += vmat(pmat, f2, c) * k;
}

/************************************************************************
 * e3: multiplica f1 por k 						*
 ************************************************************************/
 
void e3(MATRIX *pmat, int f1, double k)
{
   int c;
   
   for(c = 0; c < pmat->col; c++)
      vmat(pmat, f1, c) *= k;
}      

/************************************************************************ 
 * scandval: busca el primer valor distinto de cero desde f1 hasta 	*
 * pmat->fil traves de la columna col de la matriz. Devuelve E_NULO en 	*
 * caso de no encontrar ningun valor o la fila donde se encuentra 	*
 * el valor.								*
 ************************************************************************/
   
int scandval(MATRIX *pmat, int f1, int col)
{
   int f2;
   
   for(f2 = f1; f2 < pmat->fil; ++f2)
      if(vmat(pmat, f2, col) != 0)
         return f2;
   return E_NULO;      
}         


/************************************************************************
 * scanuval: busca el primer valor distinto de cero desde f1 hasta 0	*
 * traves de la columna col de la matriz. Devuelve E_NULO en caso de no	*
 * encontrar ningun valor o la fila donde se encuentra el valor.	*
 ************************************************************************/
   
int scanuval(MATRIX *pmat, int f1, int col)
{
   int f2;
   
   for(f2 = f1; f2 > -1; f2--)
      if(vmat(pmat, f2, col) != 0)
         return f2;
   return E_NULO;      
}         


/************************************************************************
 * scanrval: busca el primer valor distinto de cero desde c1 hasta 	*
 * pmat->col a traves de la fila fil de la matriz. Devuelve E_NULO en	*
 * caso de no encontrar ningun valor o la fila donde se encuentra el	*
 * valor.								*
 ************************************************************************/
   
int scanrval(MATRIX *pmat, int fil, int c1)
{
   int c2;
   
   for(c2 = c1; c2 < pmat->col; c2++)
      if(vmat(pmat, fil, c2) != 0)
         return c2;
   return E_NULO;      
}         

/************************************************************************
 * nullups: Busca en la matriz la fila que tiene mas cantidad de ceros	*
 * precedentes a algun elemento no nulo que la fila 'fa' y que esta por	*
 * encima de la misma. 							*
 * Regresa el numero de fila y NO_FOUND en caso de no encontrar.	*
 ************************************************************************/
 
int nullups(MATRIX *pmat, int fa)
{
   int f, c;
   int nz=0, nzfa=0; /*numero de ceros actuales y nz de fa*/
   
   for(c=0; c < pmat->col; c++){ /*ceros de la fila actual*/
      if(vmat(pmat, fa, c) != 0)
         break;
      ++nzfa;
   }
         
   for(f=0; f < fa; f++){
      for(c=0, nz=0; c < pmat->col; c++){
         if(vmat(pmat, f, c) != 0)
            break;
         ++nz;   
      }   
      if(nz > nzfa)
         return f; /*fila con mayor numero de ceros arriba de la acual*/
   }
   
   /*no hay fila con mayor numeros de ceros precedentes arriba*/
   return NO_FOUND;
}      

/************************************************************************
 * sysresolv: resuelve un sistema de n ecuaciones y n incognitas.	*
 * Almacena los valores en result, devuelve NO_SOL, si el sistema	*
 * no tiene solucion unica, M_ERROR para erro y M_OK de lo contrario.	*
 * El metodo a utilizar es el de Cramer. xn = det(xn)/det(A) .		*
 ************************************************************************/
 
int sysresolv(MATRIX *pmA, MATRIX *pmb, MATRIX *result)  
{
   MATRIX temp; /* matriz temporal */
   double detA, detsus;
   int f, c;
   
   
   /*La matriz b y result deben tener la misma catidad de filas que A*/
   if(pmA->fil != pmb->fil || pmA->fil != result->fil)
      return M_ERROR;
   
   /* primero evaluamos si el sistema tiene solucion unica */
   if((detA = det(pmA)) == 0)
      return NO_SOL; 
         
   /* matriz temporal */   
   if(creatmat(&temp, pmA->fil, 1) == NO_MEMORY){ 
      fprintf(stderr, "\nNo hay memoria disponible!\n");
      exit(EXIT_FAILURE);
   }   
   
   /*Aplicamos el metodo de Cramer*/
   
   for(c = 0; c < pmA->fil; c++){
      f = c;
      colcpy(&temp, pmA, 0, c); /* copia en temp de la columna c de pmA */
      colcpy(pmA, pmb, c, 0);
      detsus = det(pmA);
      vmat(result, f, 0) = (detsus / detA); /*valor de las incognitas*/
      colcpy(pmA, &temp, c, 0); /*volvemos a pmA con sus valores originales*/   
   } 
   
   freemat(&temp); /*Liberamos espacio temporal*/
   return M_OK;
}   
   

/************************************************************************
 * mcomp: determina el menor complementario del elemento en f,c de p.	*
 * La matriz debe ser cuadrada. Para un error su retorno no esta	*
 * definido.								*
 ************************************************************************/

double mcomp(MATRIX *p, int f, int c)
{
   int i, j; /* indices para recorrer la matriz p*/
   int tc=0, tf=0; /* indices para recorrer la matriz temp*/
   double mc; /*menor complementario*/
   
   MATRIX temp; /*matriz temporal*/
   
   if(p->col != p->fil)
      return NO_SQUARE; /* matriz no cuadrada */
   
   if(creatmat(&temp, p->fil - 1, p->col - 1) == NO_MEMORY){
        fprintf(stderr, "\nNo hay memoria disponible!\n");
        exit(EXIT_FAILURE);
   }
     
   /*copiamos el menor complementario de p a temp */
   for(i=0; i < p->fil; i++)
      for(j=0; j < p->col; j++){
         if(j == c || i == f)
            continue;
         if(tc == temp.col){
            tc = 0;
            ++tf;
         }   
         vmat(&temp, tf, tc++) = vmat(p, i, j);
      }     
         
   mc = det(&temp);
   freemat(&temp); /*liberamos espacio temporal*/
   
   return mc;      
}  

/************************************************************************
 * cof: convierte a pm1 en su matriz cofactor. Devuelve NO_SQUARE si 	*
 * hay error y M_OK si no lo hay.					*
 ************************************************************************/
  
int cof(MATRIX *pm)
{       
        
   MATRIX temp;
   int c, f;
      
   if(pm->fil != pm->col)
      return NO_SQUARE; /*matrices no cuadradas*/
   
   if(creatmat(&temp, pm->fil, pm->col) == NO_MEMORY){ /*matriz temporal*/
        fprintf(stderr, "\nNo hay memoria disponible!\n");
        exit(EXIT_FAILURE);
   }   
     
   matcpy(&temp, pm); /*copiamos pm a temp*/
   
   for(f=0; f < pm->fil; f++)
      for(c=0; c < pm->col; c++)
         vmat(pm, f, c) =  pow(-1, c+1+f+1) * mcomp(&temp, f, c); 
   
   freemat(&temp); /*liberamos matriz temporal*/      
   return M_OK;
}         

/************************************************************************
 * trn: Transpone una matriz pm. 					*
 * Si la matriz no es cuadrada, la redimenciona.			*
 ************************************************************************/   

void trn(MATRIX *pm){

   MATRIX temp;
   int f, c;
      
   if(creatmat(&temp, pm->fil, pm->col) == NO_MEMORY){ /*matriz temporal*/
      fprintf(stderr, "\nNo hay memoria disponible!\n");
      exit(EXIT_FAILURE);
   }
   
   matcpy(&temp, pm); /*copiamos pm a temp*/
      
   if(pm->fil != pm->col){ /*matriz no cuadrada, la redimencionamos*/
      freemat(pm);
      if(creatmat(pm, pm->col, pm->fil) == NO_MEMORY){ /*matriz nueva*/
         printf("\nNo hay memoria disponible!\n");
         exit(EXIT_FAILURE);
      }
   }   
  
   for(f=0; f < temp.fil; f++)
      for(c=0; c < temp.col; c++)
         vmat(pm, c, f) = vmat(&temp, f, c);
   
   freemat(&temp);      
}


/************************************************************************
 *adj: convierte a pm en su matriz adjunta, la cual por definicion es 	*
 *la transpuesta de la matriz cofactor. M_ERROR Al retornar error.		*
 ************************************************************************/

int adj(MATRIX *pm)
{

   if(cof(pm) == M_ERROR)
      return M_ERROR;
   trn(pm);
   
   return M_OK; /*simple eh :)*/
}      

/************************************************************************
 * multk: Multiplica una matriz por un escalar k			*
 ************************************************************************/
 
void multk(MATRIX *pm, double k)
{
   int f, c;
   
   for(f=0; f < pm->fil; f++)
      for(c=0; c < pm->col; c++)
         vmat(pm, f, c) *= k;
}


/************************************************************************      
 * Calcula la inversa de una matriz por el metodo de la adjunta.	*
 * inv(A) = adj(A)/det(A). Devuelve NO_SOL al no poder invertirla 	*
 * porque el det(A) = 0 (inversible) o NO_SQUARE, en caso de error.	*
 ************************************************************************/

int inv(MATRIX *pm)
{

   double detm;
   
   if(pm->fil != pm->col)
      return NO_SQUARE;
   
   if((detm = det(pm)) == 0)
      return NO_SOL; /*matriz inversible*/
      
   adj(pm); /*hacemos su adjunta*/
   multk(pm, 1/detm); /* metodo de la adjunta*/
   
   return M_OK; 
}   
      

/************************************************************************   
 * redmat: Reduce la matriz p a su reducida por filas			*
 ************************************************************************/

void redmat(MATRIX *p)
{
   
   double k;
   int c, f; /*numero de fila y col. actual */
   int c2, f2;
        
   for(c=0, f=0; c < p->col && f < p->fil; c++, f++){ 
      if(vmat(p, f, c) == 0){ /* primer elemento nulo de fila */
         if(scanrval(p, f, c+1) == E_NULO && scandval(p, f+1, c) == E_NULO)
            continue; 	/*fila nula, y por debajo en col. c solo ceros*/
         while(c < p->col){ 	/*buscamos un elemento por debajo*/
            if((f2 = scandval(p, f, c)) != E_NULO){ /*hay un elemento abajo?*/
               e1(p, f, f2); /* intercambiamos f2 por f*/  
               break;
            }
            ++c;
         }      
      }
      if(vmat(p, f, c) != 1){ /*convertimos en 1 el elemento de esta col.*/      
         k = vmat(p, f, c);
         e3(p, f, 1/k);      
      }
      if(f < (p->fil -1)) /* no estamos en la ultima fila*/
         while((f2 = scandval(p, f+1, c)) != E_NULO){ 
            k = (vmat(p, f2, c) * -1);
            e2(p, f2, f, k);
         }
      if(f > 0) /* no estamos en la primera fila*/
         while((f2 = scanuval(p, f-1, c)) != E_NULO){ 
            k = (vmat(p, f2, c) * -1);
            e2(p, f2, f, k);
         }   
      if((f2 = nullups(p, f)) != NO_FOUND){ /*la fila de arriba tiene mas ceros*/
         e1(p, f, f2); /*intercambiamos las filas*/
         f = f2;
      }   
   }    
}
   






